// **********************************************************************
//
// Copyright (c) 1999
// Object-Oriented Concepts, Inc.
// Billerica, MA, USA
//
// Copyright (c) 1999
// Object-Oriented Concepts GmbH
// Ettlingen, Germany
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OB_CODE_SET_DATABASE_H
#define OB_CODE_SET_DATABASE_H

namespace OB
{

//
// Forward declaration
//
class CodeConverterBase;

class CodeSetDatabase
{
public:

    //
    // The code set registry IDs for the supported code sets
    //
    static const CORBA::ULong ISOLATIN1;
    static const CORBA::ULong ISOLATIN2;
    static const CORBA::ULong ISOLATIN3;
    static const CORBA::ULong ISOLATIN4;
    static const CORBA::ULong ISOLATIN5;
    static const CORBA::ULong ISOLATIN6;
    static const CORBA::ULong ISOLATIN7;
    static const CORBA::ULong ISOLATIN8;
    static const CORBA::ULong ISOLATIN9;
    static const CORBA::ULong ISOLATIN0;
    static const CORBA::ULong UTF8;
    static const CORBA::ULong UTF16;

    //
    // The data required for the conversion
    //
    struct CodeSetInfo
    {
        const char* description;
	CORBA::ULong rgy_value;
	CORBA::ULong char_values_size;
	const CORBA::UShort* char_values;
	CORBA::UShort max_bytes;
    };

    //
    // The character mapping data
    //
    struct CharMapInfo
    {
	const char* name;
	CORBA::ULong lower_bound;
	CORBA::ULong upper_bound;
//	CORBA::ULong map_values_size;
	const CORBA::UShort* map_values;
    };

private:

    typedef FixSeq< CodeConverterBase*, int> CodeConverterSeq;
    CodeConverterSeq codeConverterSeq_;

    //
    // The CodeSetDatabase singleton
    //
    static OB_IMPORT CodeSetDatabase* instance_;
    friend class CodeSetDatabaseDestroyer;

    //
    // The array with code set infos and the size of that array
    //
    static const CodeSetInfo codeSetInfoArray_[];
    static const CORBA::ULong codeSetInfoArraySize_;

    //
    // The array with character mapping infos and the size of that array
    //
    static const CharMapInfo charMapInfoArray_[];
    static const CORBA::ULong charMapInfoArraySize_;

    CodeSetDatabase();
    ~CodeSetDatabase();

public:

    //
    // Get singleton instance
    //
    static CodeSetDatabase* instance();

    //
    // Get converter
    //
    CodeConverterBase* getCharEncoder(const CONV_FRAME::CodeSetComponent&,
				      CORBA::ULong);
    CodeConverterBase* getWCharEncoder(const CONV_FRAME::CodeSetComponent&,
				       CORBA::ULong);
    CodeConverterBase* getCharDecoder(const CONV_FRAME::CodeSetComponent&,
				      CORBA::ULong);
    CodeConverterBase* getWCharDecoder(const CONV_FRAME::CodeSetComponent&,
				       CORBA::ULong);

    //
    // Get CodeSetInfo for a particular registry ID
    //
    const CodeSetInfo* getCodeSetInfo(CORBA::ULong);

    //
    // Determine transmission codeset (TCS) from client/server code sets
    //
    CORBA::ULong determineTCS(const CONV_FRAME::CodeSetComponent&,
			      const CONV_FRAME::CodeSetComponent&,
			      CORBA::ULong);

    //
    // Check code set compatibility
    //
    bool isCompatible(CORBA::ULong, CORBA::ULong);

    //
    // Check for particular code set id in CodeSetComponent
    //
    bool checkCodeSetId(const CONV_FRAME::CodeSetComponent&, CORBA::ULong);

    //
    // Convert code set name to OSF code set ID and vice versa
    //
    CORBA::ULong nameToId(const char*);
    const char* idToName(CORBA::ULong);

    //
    // Get Unicode character map for a particular code set
    //
    const CharMapInfo* getCharMapInfo(const CodeSetInfo*);

    //
    // Get converter
    //
    CodeConverterBase* getConverter(const CONV_FRAME::CodeSetComponent&,
				    const CONV_FRAME::CodeSetComponent&,
				    CORBA::ULong);
};

} // End of namespace OB

#endif
